import javax.jdo.*;
import javax.jdo.annotations.Index;
import javax.persistence.Entity;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import java.util.*;

public class F2618 {

	public static void setUp() {
		// make cascading makeTransient/detach
		System.setProperty("objectdb.temp.detach-embeddable", "true");

		PersistenceManager pm = getPMF().getPersistenceManager();
		Extent<ClassWithDate> extent = pm.getExtent(ClassWithDate.class);
		pm.currentTransaction().begin();
		extent.iterator().forEachRemaining(object -> pm.deletePersistent(object));
		pm.currentTransaction().commit();
		pm.close();

		ClassWithDate instance = new ClassWithDate(
			TYPE, new java.sql.Date(System.currentTimeMillis()));
		System.out.println("ClassWithDate.date before saving: " + instance.date);
		saveToDB(instance);
	}

	private static final String TYPE = "type1";

	public static void main(String[] args) {
		setUp();
		Date date = ClassWithDate.getCorrectDate(new java.sql.Date(System.currentTimeMillis()));
		PersistenceManager pm = getPMF().getPersistenceManager();

		// these queries should find 1 item

		assertQueryResultSize(1, pm, "this.date == datum", date);
		assertQueryResultSize(1, pm, "this.type.equals(\""+ TYPE +"\") && this.date == datum", date);
		assertQueryResultSize(1, pm, "this.date.equals(datum)", date);

		// these queries should find 0 item

		assertQueryResultSize(0, pm, "this.date < datum", date);
		assertQueryResultSize(0, pm, "this.date.before(datum)", date);
		assertQueryResultSize(0, pm, "this.date > datum", date);
		assertQueryResultSize(0, pm, "this.type.equals(\""+ TYPE + "\") && this.date > datum", date);
		assertQueryResultSize(0, pm, "this.date.after(datum)", date);

		pm.close();
	}

	private static void assertQueryResultSize(int expectedResult, PersistenceManager persistenceManager,  String filter, Date date) {
		Query query = persistenceManager.newQuery(ClassWithDate.class, filter);
		query.declareParameters("java.sql.Date datum");
		Collection<ClassWithDate> collection = (Collection) query.execute(date);
		if(expectedResult != collection.size()) {
			throw new IllegalStateException("Expected: " + expectedResult + " vs actual: " + collection.size());
		}
		if(expectedResult == 1) {
			ClassWithDate instance = collection.iterator().next();
			System.out.println("Match found for filter	: " + filter);
			System.out.println("Date from DB instance	: " + instance.date);
			System.out.println("Date from filter param	: " + date);
			System.out.println("");
		}

	}

	private static void saveToDB(ClassWithDate instance) {
		PersistenceManager pm = getPMF().getPersistenceManager();
		pm.currentTransaction().begin();
		pm.makePersistent(instance);
		pm.currentTransaction().commit();
		pm.close();
	}

	private static PersistenceManagerFactory pmf;
	private static PersistenceManagerFactory getPMF() {
		if(pmf == null) {
			pmf = JDOHelper.getPersistenceManagerFactory(getConnectionProperties(), JDOHelper.class.getClassLoader());
		}
		return pmf;
	}

	private static Properties getConnectionProperties() {
		Properties props = new Properties();
		props.put("javax.jdo.PersistenceManagerFactoryClass", "com.objectdb.jdo.PMF");
		props.put("javax.jdo.option.ConnectionURL", "test2.tmp;drop");
		return props;
	}


	@Entity
	//@Unique(members = {"type", "date"})
	public static class ClassWithDate {
		@Index
		private String type;

		//@Index
		@Temporal(TemporalType.DATE)
		private java.sql.Date date;

		public ClassWithDate(String type, java.sql.Date date) {
			this.type = type;
			this.date = getCorrectDate(date);
		}

		public static java.sql.Date getCorrectDate(java.sql.Date date) {
			if (date == null) {
				return null;
			}
			Calendar calendar = Calendar.getInstance();
			//Calendar calendar = Calendar.getInstance(TimeZone.getTimeZone("UTC"));
			calendar.setTime(date);
			calendar.set(Calendar.HOUR_OF_DAY, 0);
			calendar.set(Calendar.MINUTE, 0);
			calendar.set(Calendar.SECOND, 0);
			calendar.set(Calendar.MILLISECOND, 0);
			return new java.sql.Date(calendar.getTimeInMillis());
		}

	}
}
